﻿namespace CustomDataProvider
{
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using Infragistics.Olap;
    using Infragistics.Olap.Data;
    using Infragistics.Olap.Data.Base;
    using Infragistics.Olap.Xmla;
    using CustomDataProvider.AnalysisAccess;

    public class AdomdCommand : XmlaCommand
    {
        public AdomdCommand(XmlaConnection connection)
            : base(connection)
        {
        }

        public override void ExecuteCommandAsync(IQuery query, XmlaConnection connection)
        {
            string commandText = GenerateCommand(query);
            if (this.CancelPending)
            {
                return;
            }

            AdomdConnectorClient client = new AdomdConnectorClient();

            client.ExecuteCommandCompleted += (s, args) =>
            {
                Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.OlapSchema schema =
                    new Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.OlapSchema();

                IResult result = null;

                if (args.Error == null && !args.Cancelled)
                {
                    AxisData columnAxis = null;
                    if (query.ColumnAxis != null)
                    {
                        columnAxis = args.Result.Axes[0];
                    }

                    AxisData rowAxis = null;
                    if (query.RowAxis != null)
                    {
                        rowAxis = args.Result.Axes[columnAxis == null ? 0 : 1];
                    }

                    result = GenerateResult(columnAxis, rowAxis, args.Result.Cells);
                }

                this.OnExecuteCommandCompleted(new Infragistics.Olap.ExecuteCommandCompletedEventArgs(result, args.Error, args.Cancelled, query));
            };

            client.ExecuteCommandAsync(((AdomdConnectionSettings)connection.ConnectionSettings).ConnectionString, commandText);
        }

        private static Result GenerateResult(AxisData columnAxis, AxisData rowAxis, IList<CellData> cellsData)
        {
            ResultAxis resultColumnAxis = GenerateAxis(columnAxis);

            ResultAxis resultRowAxis = GenerateAxis(rowAxis);

            ICell[,] cells = new ICell[resultRowAxis.Tuples.Count, resultColumnAxis.Tuples.Count];
            FillCellData(cells, cellsData);

            return new Result(resultRowAxis, resultColumnAxis, cells);
        }

        private static void FillCellData(ICell[,] cells, IList<CellData> cellsData)
        {
            if (cellsData == null)
            {
                return;
            }

            int columnsCount = cells.GetLength(1);

            for (int i = 0; i < cellsData.Count; i++)
            {
                CellData cellData = cellsData[i];
                int columnNumber = i % columnsCount;
                int rowNumber = i / columnsCount;

                cells[rowNumber, columnNumber] = new Cell(cellData.Value, cellData.FormattedValue);
            }
        }

        private static ResultAxis GenerateAxis(AxisData axis)
        {
            List<ITuple> tuples = new List<ITuple>();
            int[] maxDepths;

            if (axis == null)
            {
                List<IMember> members = new List<IMember>();

                Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Tuple axisTuple =
                    new Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Tuple(new OlapElementCollection<IMember>(members));

                Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Member member =
                            new Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Member(string.Empty, string.Empty, string.Empty, string.Empty, string.Empty, 0, null, 0, false, false, false);
                members.Add(member);

                tuples.Add(axisTuple);

                maxDepths = new[] { 1 };
            }
            else
            {
                PositionData prevTuple = null;
                foreach (PositionData positionData in axis.Positions)
                {
                    List<IMember> members = new List<IMember>();
                    int memberIndex = 0;

                    foreach (MemberData memberData in positionData.Members)
                    {
                        bool isTotal = false;
                        if (memberData.DrilledDown)
                        {
                            // member.DrillDownMember = true;
                            isTotal = true;
                        }
                        else
                        {
                            // the member from the previous tuple is child of this member
                            // that does mean it is the Total column and it follows its children members                          
                            if (!memberData.ParentSameAsPrevious &&
                                prevTuple != null &&
                                prevTuple.Members[memberIndex].ParentUniqueName == memberData.UniqueName)
                            {
                                isTotal = true;
                            }
                        }

                        Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Member member =
                            new Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Member(
                            memberData.Name,
                            memberData.UniqueName,
                            memberData.Caption,
                            memberData.ParentUniqueName,
                            memberData.LevelName,
                            memberData.LevelDepth,
                            null, // TODO : Find if we can set parent level
                            memberData.ChildCount,
                            memberData.DrilledDown,
                            memberData.ParentSameAsPrevious,
                            isTotal);

                        members.Add(member);

                        memberIndex++;
                    }

                    prevTuple = positionData;

                    Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Tuple axisTuple =
                        new Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Tuple(new OlapElementCollection<IMember>(members));

                    tuples.Add(axisTuple);
                }

                if (axis.Positions.Count > 0)
                {
                    int count = axis.Positions[0].Members.Count;
                    maxDepths = new int[count];

                    for (int i = 0; i < count; i++)
                    {
                        foreach (ITuple tuple in tuples)
                        {
                            if (tuple.Members[i].LevelDepth >= maxDepths[i])
                            {
                                maxDepths[i] = tuple.Members[i].LevelDepth + 1;
                            }
                        }
                    }

                    int maxDepth = 0;
                    for (int i = 0; i < maxDepths.Length; i++)
                    {
                        int depth = maxDepths[i];
                        maxDepth += depth;
                        maxDepths[i] = maxDepth;
                    }
                }
                else
                {
                    maxDepths = new[] { 1 };
                }
            }

            return new ResultAxis(new ReadOnlyCollection<ITuple>(tuples), maxDepths);
        }
    }
}
